from .exit_if_empty import exit_if_empty
from .input_char import input_char
from .print_colorize import print_colorize


def print_choices(lists):
    for i, v in enumerate(lists):
        print(f"{i}. {v}")


def get_choices(prompt):
    print_colorize(prompt, text_end="")
    i = input_char()
    print()
    return i


def convert_to_str(daftar):
    if len(daftar) == 1:
        daftar = daftar[0]
    return daftar


def filter_choices(contains, daftar):
    r = []
    if contains.isdigit():
        for i, v in enumerate(daftar):
            if contains in str(i):
                r.append(v)
        # daftar = [v for i, v in enumerate(daftar) if contains in str(i)]
    else:
        for v in daftar:
            if contains in v:
                r.append(v)
        # daftar = [v for v in daftar if contains in v]
    return r


def get_contains(daftar, prompt):
    print_choices(daftar)
    contains = get_choices(prompt)
    exit_if_empty(len(contains))
    return contains


def choices(daftar, contains=None, prompt="Choose : "):
    """
    Memudahkan dalam membuat pilihan untuk user dalam tampilan console

    ```py
    var = {
        "Pertama" : "Pilihan Pertama",
        "Kedua" : "Pilihan Kedua",
        "Ketiga" : "Pilihan Ketiga",
    }
    res = choices(
        var,
        prompt="Pilih dari dictionary : "
    )
    print(res)
    ```
    """
    daftar = list(str(v) for v in daftar)
    while isinstance(daftar, list):
        contains = contains or get_contains(daftar, prompt)
        daftar = filter_choices(contains, daftar)
        exit_if_empty(daftar)
        daftar = convert_to_str(daftar)
        contains = None
    return daftar



import subprocess

from .print_log import print_log


def console_run(info, command=None, print_info=True, capture_output=False):
    """
    Menjalankan command seperti menjalankan command di Command Terminal

    ```py
    console_run('dir')
    console_run('ls')
    ```
    """
    if command is None:
        command = info

    if print_info:
        print_log(info)

    param = dict(shell=True)
    if capture_output:
        param |= dict(capture_output=True, text=True)

    return subprocess.run(command, **param)



import os
import shutil
from datetime import datetime


def sqlite_backup(db):
    # db = os.path.abspath(db)
    if not os.path.isfile(db):
        raise FileNotFoundError(f"File '{db}' tidak ditemukan.")

    ff, ext = os.path.splitext(os.path.basename(db))

    fdb = os.path.join(os.path.dirname(db), ff)
    os.makedirs(fdb, exist_ok=True)

    nff = f"{ff}_{datetime.now():%Y%m%d_%H%M%S}{ext}"
    ndb = os.path.join(fdb, nff)

    shutil.copy2(db, ndb)
    return ndb



def get_by_index(obj, index, on_error=None):
    """
    Mendapatkan value dari object berdasarkan indexnya.
    Jika error out of range maka akan mengembalikan on_error.
    
    ```python
    l = [1, 3, 5]
    print(get_by_index(l, 7))
    ```
    """
    try:
        return obj[index]
    except Exception:
        return on_error



class TextCase:
    simbol = ".-_/\ "
    kapital = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"

    def __init__(self, text: str) -> None:
        self.text = self.explode(text)
        self.double_symbol = True

    def explode(self, text):
        r = []
        b = ""

        for t in text:
            if t in self.simbol:
                if len(b):
                    r.append(b)
                    b = ""
                r.append(t)
            elif t in self.kapital:
                if len(b):
                    r.append(b)
                    b = ""
                b = t
            else:
                b += t

        if len(b):
            r.append(b)
            b = ""

        return r

    def to_title_case(self):
        r = ""
        z = True
        for i in self.text:
            if i in self.simbol:
                if not z or self.double_symbol:
                    r += ""
                z = True
            else:
                if not z:
                    r += ""
                r += i.title()
                z = False
        return r

    def to_snake_case(self):
        r = ""
        z = True
        for i in self.text:
            if i in self.simbol:
                if not z or self.double_symbol:
                    r += "_"
                z = True
            else:
                if not z:
                    r += "_"
                r += i.lower()
                z = False
        return r

    def to_camel_case(self):
        r = ""
        z = True
        for i in self.text:
            if i in self.simbol:
                if not z or self.double_symbol:
                    r += ""
                z = True
            else:
                if not z:
                    r += ""
                r += i.title()
                z = False
        return r

    def to_path_case(self):
        r = ""
        z = True
        for i in self.text:
            if i in self.simbol:
                if not z or self.double_symbol:
                    r += "/"
                z = True
            else:
                if not z:
                    r += "/"
                r += i
                z = False
        return r

    def to_dot_case(self):
        r = ""
        z = True
        for i in self.text:
            if i in self.simbol:
                if not z or self.double_symbol:
                    r += "."
                z = True
            else:
                if not z:
                    r += "."
                r += i
                z = False
        return r

    def to_space_case(self):
        r = ""
        z = True
        for i in self.text:
            if i in self.simbol:
                if not z or self.double_symbol:
                    r += " "
                z = True
            else:
                if not z:
                    r += " "
                r += i
                z = False
        return r



def int_to_int(n):
    """
    Fungsi ini sama seperti fungsi int().
    fungsi ini dibuat hanya untuk keperluan pembuatan module semata.

    ```python
    print(int_to_int(7777))
    ```
    """
    return int(n)



import os


def dirname(path):
    """
    Mengembalikan nama folder dari path.
    Tanpa trailing slash di akhir.

    ```python
    print(dirname("/ini/nama/folder/ke/file.py"))
    ```
    """
    return os.path.dirname(path)



from .print_colorize import print_colorize


def print_log(text):
    """
    Akan melakukan print ke console.
    Berguna untuk memberikan informasi proses program yg sedang berjalan.

    ```python
    print_log("Standalone Log")
    ```
    """
    print_colorize(f">>> {text}")



# import subprocess
import time

from .get_filemtime import get_filemtime
from .print_log import print_log
from .console_run import console_run


def auto_reload(filename):
    """
    Menjalankan file python secara berulang.
    Dengan tujuan untuk melihat perubahan secara langsung.
    Pastikan kode aman untuk dijalankan.
    Jalankan kode ini di terminal console.

    ```py
    auto_reload("file_name.py")
    ```

    or run in terminal

    ```
    pypipr auto_reload
    ```
    """
    mtime = get_filemtime(filename)
    last_mtime = 0
    cmd = f"python {filename}"

    try:
        print_log("Start")
        while True:
            last_mtime = mtime
            console_run(cmd, print_info=False)
            while mtime == last_mtime:
                time.sleep(1)
                mtime = get_filemtime(filename)
            print_log("Reload")
    except KeyboardInterrupt:
        print_log("Stop")



import math
import pprint

import colorama

from .is_iterable import is_iterable


def iprint(
    *args,
    color=None,
    sort_dicts=False,
    **kwargs,
):
    """
    Improve print function dengan menambahkan color dan pretty print
    Color menggunakan colorama Fore + Back + Style

    ```python
    import colorama
    iprint(
        'yang ini',
        {'12':12,'sdsd':{'12':21,'as':[88]}},
        color=colorama.Fore.BLUE + colorama.Style.BRIGHT
    )
    ```
    """

    r = []
    for i in args:
        if is_iterable(i) and not isinstance(i, (list, set, dict, tuple)):
            i = list(i)

        if not isinstance(i, str):
            i = pprint.pformat(i, depth=math.inf, sort_dicts=sort_dicts)

        if color:
            i = color + i + colorama.Style.RESET_ALL

        r.append(i)

    print(*r, **kwargs)



from pathlib import Path


def repath(
    path: Path,
    folder_name=None,
    prepand_folder=None,
    append_folder=None,
    file_name=None,
    prepand_filename=None,
    append_filename=None,
    extension=None,
    prepand_extension=None,
    append_extension=None,
):
    # Handle the directory (folder) part
    parent = path.parent
    if folder_name is not None:
        parent = Path(folder_name)
    if prepand_folder is not None:
        parent = Path(prepand_folder) / parent
    if append_folder is not None:
        parent = parent / append_folder

    # Handle the file name part
    name = path.stem  # Gets the file name without the extension
    if file_name is not None:
        name = file_name
    if prepand_filename is not None:
        name = prepand_filename + name
    if append_filename is not None:
        name = name + append_filename

    # Handle the extension part
    ext = path.suffix  # Gets the file extension (including the dot)
    if extension is not None:
        ext = extension
    if prepand_extension is not None:
        ext = prepand_extension + ext
    if append_extension is not None:
        ext = ext + append_extension

    # Combine everything into a new path
    new_path = parent / (name + ext)

    return new_path



import colorama

from .is_empty import is_empty
from .print_colorize import print_colorize


def print_dir(var, colorize=True):
    """
    Print property dan method yang tersedia pada variabel

    ```python
    import pathlib
    p = pathlib.Path("https://www.google.com/")
    print_dir(p, colorize=False)
    ```
    """
    d = dir(var)
    m = max(len(i) for i in d)
    for i in d:
        try:
            a = getattr(var, i)
            r = a() if callable(a) else a
            if colorize:
                color = colorama.Fore.GREEN
                if is_empty(r):
                    color = colorama.Fore.LIGHTRED_EX
                if i.startswith("__"):
                    color = colorama.Fore.CYAN
                print_colorize(f"{i: >{m}}", text_end=" : ", color=color)
                print(r)
            else:
                print(f"{i: >{m}} : {r}")
        except Exception:
            pass



import colorama


def text_colorize(
    text,
    color=colorama.Fore.GREEN,
    bright=colorama.Style.BRIGHT,
    color_end=colorama.Style.RESET_ALL,
):
    """
    return text dengan warna untuk menunjukan text penting

    ```py
    text_colorize("Print some text")
    text_colorize("Print some text", color=colorama.Fore.RED)
    ```
    """
    return f"{color + bright}{text}{color_end}"



import pint

"""
Class Pint Unit Registry dasar untuk membuat Pint Object
Setiap instance Unit Registry hanya bisa dilakukan operasi perhitungan
dengan instance Unit Registry itu sendiri
"""
PintUreg = pint.UnitRegistry()



from .print_colorize import print_colorize
from .LINUX import LINUX
from .WINDOWS import WINDOWS

if WINDOWS:
    import msvcrt as getch

if LINUX:
    import getch as getch


def input_char(
    prompt=None,
    prompt_ending="",
    newline_after_input=True,
    echo_char=True,
    default=None,
    color=None,
):
    """
    Meminta masukan satu huruf tanpa menekan Enter.

    ```py
    input_char("Input char : ")
    input_char("Input char : ", default='Y')
    input_char("Input Char without print : ", echo_char=False)
    ```
    """
    if prompt:
        print_colorize(prompt, end=prompt_ending, flush=True)

    if default is not None:
        a = default
    else:
        a = getch.getche() if echo_char else getch.getch()

    if newline_after_input:
        print()

    if WINDOWS:
        return a.decode()
    if LINUX:
        return a
    raise Exception("Platform tidak didukung.")



from .batchmaker import batchmaker
from .calculate import calculate


def batch_calculate(pattern):
    """
    Analisa perhitungan massal.
    Bisa digunakan untuk mencari alternatif terendah/tertinggi/dsb.


    ```python
    print(batch_calculate("{1 10} m ** {1 3}"))
    print(list(batch_calculate("{1 10} m ** {1 3}")))
    ```
    """
    patterns = batchmaker(pattern)
    for i in patterns:
        try:
            yield (i, calculate(i))
        except Exception:
            yield (i, None)



from pathlib import Path
from time import sleep


def create_folder(folder_name, wait_until_success=True):
    """
    Membuat folder.
    Membuat folder secara recursive dengan permission.

    ```py
    create_folder("contoh_membuat_folder")
    create_folder("contoh/membuat/folder/recursive")
    create_folder("./contoh_membuat_folder/secara/recursive")
    ```
    """
    p = Path(folder_name)
    p.mkdir(parents=True, exist_ok=True)
    if wait_until_success:
        while not p.exists():
            sleep(0.1)

